package de.intevation.flys.client.client.ui;

import com.google.gwt.core.client.GWT;

import com.google.gwt.user.client.rpc.AsyncCallback;

import com.smartgwt.client.util.SC;

import com.smartgwt.client.widgets.Window;

import com.smartgwt.client.widgets.tree.Tree;
import com.smartgwt.client.widgets.tree.TreeGrid;
import com.smartgwt.client.widgets.tree.TreeNode;

import com.smartgwt.client.types.TreeModelType;

import com.smartgwt.client.widgets.layout.Layout;
import com.smartgwt.client.widgets.layout.VLayout;

import com.smartgwt.client.widgets.tree.events.LeafClickEvent;
import com.smartgwt.client.widgets.tree.events.LeafClickHandler;

import de.intevation.flys.client.shared.model.Artifact;
import de.intevation.flys.client.shared.model.ArtifactDescription;
import de.intevation.flys.client.shared.model.Data;
import de.intevation.flys.client.shared.model.DataList;
import de.intevation.flys.client.shared.model.DataItem;
import de.intevation.flys.client.shared.model.User;

import de.intevation.flys.client.client.FLYSConstants;
import de.intevation.flys.client.client.Config;

import de.intevation.flys.client.client.services.MetaDataService;
import de.intevation.flys.client.client.services.MetaDataServiceAsync;

import de.intevation.flys.client.shared.model.DataCageTree;
import de.intevation.flys.client.shared.model.DataCageNode;
import de.intevation.flys.client.shared.model.AttrList;

import java.util.ArrayList;
import java.util.List;

public class DatacageWindow
extends      Window
{
    protected MetaDataServiceAsync metaDataService =
        GWT.create(MetaDataService.class);

    protected FLYSConstants messages =
        GWT.create(FLYSConstants.class);


    protected Artifact artifact;
    protected User     user;

    protected TreeGrid treeGrid;
    protected Tree     tree;

    protected Layout layout;


    public DatacageWindow(Artifact artifact, User user) {
        this.artifact = artifact;
        this.user     = user;

        setWidth(250);
        setHeight(400);

        layout = new VLayout();
        layout.setWidth100();

        addItem(layout);

        tree = new Tree();
        tree.setModelType(TreeModelType.PARENT);
        tree.setNameProperty("name");
        tree.setIdField("id");
        tree.setParentIdField("parent-id");

        treeGrid = new TreeGrid();
        treeGrid.setLoadDataOnDemand(false);
        treeGrid.setWidth100();
        treeGrid.setHeight100();
        treeGrid.setShowRoot(false);

        treeGrid.addLeafClickHandler(new LeafClickHandler() {
            @Override
            public void onLeafClick(LeafClickEvent lce) {
                GWT.log("I was here");
            }
        });

        layout.addMember(treeGrid);

        String river =  findRiver();
        // TODO: i18n
        setTitle("Datenkorb: " + river);
        setShowMinimizeButton(false);
        setIsModal(true);
        setShowModalMask(true);
        setCanDragResize(true);

        centerInPage();

        triggerTreeBuilding();
    }

    protected void triggerTreeBuilding() {
        Config config = Config.getInstance();
        String url    = config.getServerUrl();
        String locale = config.getLocale();

        String artifactId = artifact.getUuid();
        String userId     = user != null ? user.identifier() : null;

        metaDataService.getMetaData(
            url, locale,
            artifactId,
            userId,
            new AsyncCallback<DataCageTree>() {
                public void onFailure(Throwable caught) {
                    GWT.log("Could not load meta data.");
                    SC.warn(caught.getMessage());
                }

                public void onSuccess(DataCageTree dcTree) {
                    GWT.log("Successfully loaded meta data.");
                    tree.setData(treeToData(dcTree));
                    treeGrid.setData(tree);
                }
            });
    }

    private static class IdGenerator {
        protected int current;

        public IdGenerator() {
        }

        public IdGenerator(int start) {
            current = start;
        }

        public int next() {
            return current++;
        }
    } // class IdGenerator

    protected static void buildRecursive(
        DataCageNode   node, 
        int            parentId,
        IdGenerator    idGenerator,
        List<TreeNode> nodes
    ) {
        TreeNode tn = new TreeNode();
        int id = idGenerator.next();
        tn.setAttribute("parent-id", parentId);
        tn.setAttribute("id", id);
        // TODO: i18n
        tn.setAttribute("name", node.getName());
        nodes.add(tn);
        AttrList attrs = node.getAttributes();
        if (attrs != null) {
            for (int i = 0, N = attrs.size(); i < N; ++i) {
                String key   = attrs.getKey(i);
                String value = attrs.getValue(i);
                tn.setAttribute(key, value);
            }
        }
        List<DataCageNode> children = node.getChildren();
        if (children != null) {
            for (DataCageNode child: children) {
                buildRecursive(child, id, idGenerator, nodes);
            }
        }
    }


    protected static TreeNode [] treeToData(DataCageTree tree) {
        List<TreeNode> nodes = new ArrayList<TreeNode>();

        DataCageNode root = tree.getRoot();

        IdGenerator idGenerator = new IdGenerator();

        buildRecursive(root, idGenerator.next(), idGenerator, nodes);

        return nodes.toArray(new TreeNode[nodes.size()]);
    }

    protected String findRiver() {

        ArtifactDescription adescr = artifact.getArtifactDescription();
        DataList [] data = adescr.getOldData();

        if (data != null && data.length > 0) {
            for (int i = 0; i < data.length; i++) {
                DataList dl = data[i];
                if (dl.getState().equals("state.winfo.river")) {
                    for (int j = dl.size()-1; j >= 0; --j) {
                        Data d = dl.get(j);
                        DataItem [] di = d.getItems();
                        if (di != null && di.length == 1) {
                           return d.getItems()[0].getStringValue();
                        }
                    }
                }
            }
        }

        return "";
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
